@@ -3,6 +3,7 @@
 /*
 Copyright (C) 2005 Jean-Baptiste jb_dul@yahoo.com
 Copyright (C) 2005-2008 Fabian Nowak timystery@arcor.de.
+Copyright (C) 2009,2012 Landry Breuil <landry@xfce.org>
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -20,20 +21,36 @@ along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <fstab.h>
 #include <glib.h>
-#include <mntent.h>
 #include <stdio.h>
 #include <string.h>
+/* WEXITSTATUS */
+#include <sys/types.h>
+#include <sys/wait.h>
+#ifdef HAVE_GETMNTENT
+#include <mntent.h>
 #include <sys/vfs.h>
+#elif defined (HAVE_GETMNTINFO)
+#include <sys/param.h>
+#include <sys/mount.h>
+#else
+#error no getmntent/getmntinfo ? send patches !
+#endif
 
 #include <libxfce4panel/xfce-panel-plugin.h>
 #include <libxfce4util/libxfce4util.h>
-#include <libxfcegui4/xfce-exec.h>
-#include <libxfcegui4/dialogs.h>
+#include <libxfce4ui/libxfce4ui.h>
 
 #include "devices.h"
 
+/* declare that one here to appease -Werror */
+t_mount_info * mount_info_new_from_stat (struct statfs * pstatfs, char * mnt_type, char * mnt_dir);
+
 #define KB 1024
 #define MB 1048576
 #define GB 1073741824
@@ -162,6 +179,21 @@ disk_print (t_disk * pdisk)
     return;
 }
 
+char *
+shorten_disk_name (const char *dev)
+{
+    char *r, *lastchars;
+    if (strncmp(dev, "UUID", 4)==0 && strlen(dev)>13)
+    {
+        lastchars = (char *) (dev + strlen(dev) - 5);
+        r = (char *) malloc (14*sizeof(char));
+        snprintf (r, 14, "UUID=...%s", lastchars);
+    }
+    else
+        r = g_strdup (dev);
+
+    return r;
+}
 
 t_disk *
 disk_new (const char * dev, const char * mountpoint)
@@ -173,7 +205,7 @@ disk_new (const char * dev, const char * mountpoint)
     if ( dev != NULL && mountpoint != NULL)
     {
         pdisk = g_new0 (t_disk,1);
-        pdisk->device = g_strdup(dev);
+        pdisk->device = shorten_disk_name (dev);
         pdisk->mount_point = g_strdup (mountpoint);
         pdisk->mount_info = NULL;
 
@@ -204,49 +236,71 @@ disk_free(t_disk **pdisk)
  * Return exit status of the mount command
  */
 void
-disk_mount (t_disk *pdisk, char *on_mount_cmd, char* mount_command, gboolean eject)
+disk_mount (t_disk *pdisk, char *on_mount_cmd, char* mount_command, gboolean eject, gboolean use_sudo)
 {
-    gchar *tmp=NULL, *cmd, *tmp2 = NULL;
+    gchar *tmp = NULL, *cmd = NULL;
+    gchar *output = NULL, *erroutput = NULL;
+    int exit_status = 0;
     GError *error = NULL;
     gboolean val;
 
-    #ifdef DEBUG
-    g_printf ("disk_mount: eject=%d\n", eject);
-    #endif
-
     if (pdisk != NULL)
     {
+        DBG("disk_mount: dev=%s, mountpoint=%s, mount_command=%s, on_mount_cmd=%s, eject=%d, use_sudo=%d", pdisk->device, pdisk->mount_point, mount_command, on_mount_cmd, eject, use_sudo);
+        if (eject) {
+#ifdef __OpenBSD__
+/* hack: on OpenBSD, eject(1) -t expects cd0/cd1 (or rcd0c/rcd1c), if passed /dev/cdXa it will spit 'No medium found' */
+            tmp = g_strstr_len(pdisk->device, strlen(pdisk->device), "/dev/cd");
+            if (tmp) {
+                cmd = g_strconcat ("eject -t cd", tmp + 7, NULL);
+                /* remove chars after cdX */
+                cmd[12] = '\0';
+                tmp = NULL;
+            }
+            else
+                cmd = g_strconcat ("eject -t ", pdisk->device, NULL);
+#else
+            cmd = g_strconcat ("eject -t ", pdisk->device, NULL);
+#endif
+            val = g_spawn_command_line_sync (cmd, &output, &erroutput, &exit_status, &error);
+            DBG("cmd: '%s', returned %d, exit_status=%d", cmd, val, exit_status);
+            if (val == FALSE || exit_status != 0)
+               goto out;
+            g_free(cmd);
+            cmd = NULL;
+        }
         deviceprintf (&tmp, mount_command, pdisk->device);
-        mountpointprintf (&tmp2, tmp, pdisk->mount_point);
-        if (eject)
-            cmd = g_strconcat ("sh -c ' eject -t ", pdisk->device, " && ", tmp2, NULL);
-        else
-            cmd = g_strconcat ("sh -c ' ", tmp2, NULL);
-
-        g_free(tmp);
-        g_free(tmp2);
-        tmp = NULL;
-        tmp2 = NULL;
+        mountpointprintf (&cmd, tmp, pdisk->mount_point);
+        /* cmd contains mount_command device mount_point */
+        if (use_sudo) {
+            g_free(tmp);
+            tmp = g_strdup(cmd);
+            g_free(cmd);
+            cmd = g_strconcat ("sudo ", tmp, NULL);
+        }
+        val = g_spawn_command_line_sync (cmd, &output, &erroutput, &exit_status, &error);
+        DBG("cmd: '%s', returned %d, exit_status=%d", cmd, val, exit_status);
+        if (val == FALSE || exit_status != 0)
+           goto out;
 
         if (on_mount_cmd != NULL && strlen(on_mount_cmd)!=0) {
+            g_free(tmp);
+            tmp = NULL;
+            g_free(cmd);
+            cmd = NULL;
             deviceprintf(&tmp, on_mount_cmd, pdisk->device);
-            mountpointprintf(&tmp2, tmp, pdisk->mount_point);
-            cmd = g_strconcat (cmd, " && ", tmp2, " '", NULL);
+            mountpointprintf(&cmd, tmp, pdisk->mount_point);
+            val = g_spawn_command_line_sync (cmd, &output, &erroutput, &exit_status, &error);
+            DBG("cmd: '%s', returned %d, exit_status=%d", cmd, val, exit_status);
         }
-        else
-            cmd = g_strconcat (cmd, " '", NULL);
-
-        #ifdef DEBUG
-        g_printf ("cmd: '%s'\n", cmd);
-        #endif
-
-        val = xfce_exec (cmd, FALSE, FALSE, &error);
-        if (!val)
-            xfce_err (_("Mount Plugin: Error executing command."));
-
+out:
         g_free(cmd);
-        g_free(tmp);
-        g_free(tmp2);
+        if (tmp)
+            g_free(tmp);
+        /* show error message if smth failed */
+        if (val == FALSE || exit_status != 0)
+            xfce_dialog_show_error (NULL, error, "%s %s %d, %s %s", _("Mount Plugin: Error executing command."),
+                _("Returned"), WEXITSTATUS(exit_status), _("error was"), erroutput);
     }
 }
 
@@ -254,56 +308,55 @@ disk_mount (t_disk *pdisk, char *on_mount_cmd, char* mount_command, gboolean eje
 /**
  * Return exit status of the umount command.
  */
-int
-disk_umount (t_disk *pdisk, char* umount_command, gboolean synchronous, gboolean eject)
+void
+disk_umount (t_disk *pdisk, char* umount_command, gboolean show_message_dialog, gboolean eject, gboolean use_sudo)
 {
-    int retval = NONE;
-    gchar *tmp = NULL, *tmp2 = NULL;
-    gboolean val;
-    char *cmd;
+    gchar *tmp = NULL, *cmd = NULL;
+    gchar *output = NULL, *erroutput = NULL;
+    int exit_status = 0;
     GError *error = NULL;
-
-    #ifdef DEBUG
-    g_printf ("disk_umount: eject=%d\n", eject);
-    #endif
-
+    gboolean val;
 
     if (pdisk != NULL)
     {
 
+        DBG("disk_umount: dev=%s, mountpoint=%s, umount_command=%s, show_message_dialog=%d, eject=%d, use_sudo=%d", pdisk->device, pdisk->mount_point, umount_command, show_message_dialog, eject, use_sudo);
         deviceprintf(&tmp, umount_command, pdisk->device);
-        mountpointprintf(&tmp2, tmp, pdisk->mount_point);
-        cmd = g_strconcat ("sh -c '", tmp2, NULL);
+        mountpointprintf(&cmd, tmp, pdisk->mount_point);
+        /* cmd contains umount_command device mount_point */
+        if (use_sudo) {
+            g_free(tmp);
+            tmp = g_strdup(cmd);
+            g_free(cmd);
+            cmd = g_strconcat ("sudo ", tmp, NULL);
+        }
+        val = g_spawn_command_line_sync (cmd, &output, &erroutput, &exit_status, &error);
+        DBG("cmd: '%s', returned %d, exit_status=%d", cmd, val, exit_status);
+        if (val == FALSE || exit_status != 0)
+           goto out;
 
-        /* re-use tmp */
-        g_free(tmp);
-        tmp = NULL;
-
-        if (eject)
-            tmp = g_strconcat (cmd, " && eject ", pdisk->device, " '", NULL);
-        else
-            tmp = g_strconcat (cmd, " '", NULL);
-
-        #ifdef DEBUG
-        g_printf ("cmd: '%s'\n", tmp);
-        #endif
-
-        val = xfce_exec_sync (tmp, FALSE, FALSE, &error);
-
-        if  (!val) {
-            xfce_err (_("Mount Plugin: Error executing command."));
-            retval = ERROR;
+        if (eject) {
+            g_free(cmd);
+            cmd = NULL;
+            cmd = g_strconcat ("eject ", pdisk->device, NULL);
+            val = g_spawn_command_line_sync (cmd, &output, &erroutput, &exit_status, &error);
+            DBG("cmd: '%s', returned %d, exit_status=%d", cmd, val, exit_status);
         }
 
-        if (disk_check_mounted(pdisk->device))
-            retval = ERROR;
+out:
+        g_free(cmd);
+        if (tmp)
+            g_free(tmp);
+        /* show error message if smth failed */
+        if (val == FALSE || exit_status != 0)
+            xfce_dialog_show_error (NULL, error, "%s %s %d, %s %s", _("Mount Plugin: Error executing command."),
+                _("Returned"), WEXITSTATUS(exit_status), _("error was"), erroutput);
 
-        g_free (cmd);
-        g_free (tmp);
-        g_free (tmp2);
+        if (show_message_dialog && !eject && val == TRUE && exit_status == 0)
+            xfce_dialog_show_info (NULL, NULL, _("The device \"%s\" should be removable safely now."), pdisk->device);
+        if (show_message_dialog && disk_check_mounted(pdisk->device))
+            xfce_dialog_show_error (NULL, NULL, _("An error occurred. The device \"%s\" should not be removed!"), pdisk->device);
     }
-
-    return retval;
 }
 
 
@@ -315,18 +368,34 @@ disk_umount (t_disk *pdisk, char* umount_command, gboolean synchronous, gboolean
  * @return                GPtrArray *pdisks containing elements to be displayed
  */
 GPtrArray *
-disks_new (gboolean include_NFSs)
+disks_new (gboolean include_NFSs, gboolean *showed_fstab_dialog)
 {
     GPtrArray * pdisks; /* to be returned */
     t_disk * pdisk;
     struct fstab *pfstab;
     gboolean has_valid_mount_device;
+    GtkWidget *dialog;
+
+    pdisks = g_ptr_array_new();
 
     /* open fstab */
     if (setfsent()!=1)
-        return NULL; /* on error return NULL */
+    {
+        /* popup notification dialog */
+        if (! (*showed_fstab_dialog) ) {
+            dialog = gtk_message_dialog_new (NULL, GTK_DIALOG_DESTROY_WITH_PARENT,
+                                                GTK_MESSAGE_INFO, GTK_BUTTONS_OK,
+    "Your /etc/fstab could not be read. This will severely degrade the plugin's abilities.");
+            /* gtk_dialog_run (GTK_DIALOG (dialog)); */
+            g_signal_connect (dialog, "response",
+                    G_CALLBACK (gtk_widget_destroy), dialog);
+             gtk_widget_show (dialog);
+             *showed_fstab_dialog = TRUE;
+         }
+
+        return pdisks; /* on error return empty pdisks */
+    }
 
-    pdisks = g_ptr_array_new();
 
     for (pfstab = getfsent(); pfstab!=NULL; pfstab = getfsent())
     {
@@ -363,7 +432,7 @@ disks_new (gboolean include_NFSs)
 void
 disks_free (GPtrArray ** pdisks)
 {
-    int i ;
+    unsigned int i ;
     t_disk * pdisk ;
 
     if (*pdisks != NULL)
@@ -385,7 +454,7 @@ disks_free (GPtrArray ** pdisks)
 void
 disks_print (GPtrArray * pdisks)
 {
-    int i ;
+    unsigned int i ;
     for (i=0; i < pdisks->len ; i++)
     {
         disk_print (g_ptr_array_index(pdisks, i));
@@ -402,7 +471,7 @@ disks_print (GPtrArray * pdisks)
 gboolean
 disks_remove_device (GPtrArray * pdisks, char *device)
 {
-    int i;
+    unsigned int i;
     gpointer p=NULL;
 
     for (i=0; i < pdisks->len ; i++)
@@ -426,7 +495,7 @@ disks_remove_device (GPtrArray * pdisks, char *device)
 gboolean
 disks_remove_mountpoint (GPtrArray * pdisks, char *mountp)
 {
-    int i;
+    unsigned int i;
     gpointer p=NULL;
 
     for (i=0; i < pdisks->len ; i++)
@@ -450,7 +519,7 @@ disks_remove_mountpoint (GPtrArray * pdisks, char *mountp)
 t_disk *
 disks_search (GPtrArray * pdisks, char * mount_point)
 {
-    int i ;
+    unsigned int i ;
 
     for (i=0; i < pdisks->len ; i++)
     {
@@ -471,7 +540,7 @@ disks_search (GPtrArray * pdisks, char * mount_point)
 void
 disks_free_mount_info(GPtrArray * pdisks)
 {
-    int i ;
+    unsigned int i ;
 
     for (i=0; i < pdisks->len ; i++)
     {
@@ -488,15 +557,15 @@ disks_free_mount_info(GPtrArray * pdisks)
 gboolean
 exclude_filesystem (GPtrArray *excluded_FSs, gchar *mountpoint, gchar *device)
 {
-    int i;
+    unsigned int i;
 
-    TRACE("Entering exclude_filesystems\n");
+    TRACE("Entering exclude_filesystems");
 
     g_assert(excluded_FSs != NULL);
 
     for (i=0; i < excluded_FSs->len; i++)
     {
-        DBG("Comparing %s and %s to %s\n", mountpoint, device, (gchar *) g_ptr_array_index(excluded_FSs, i));
+        DBG("Comparing %s and %s to %s", mountpoint, device, (gchar *) g_ptr_array_index(excluded_FSs, i));
         if (g_ascii_strcasecmp (
                 (gchar *) g_ptr_array_index(excluded_FSs, i), mountpoint)==0
             ||
@@ -506,7 +575,7 @@ exclude_filesystem (GPtrArray *excluded_FSs, gchar *mountpoint, gchar *device)
             return TRUE;
     }
 
-    TRACE("Leaving exclude_filesystems with FALSE\n");
+    TRACE("Leaving exclude_filesystems with FALSE");
 
     return FALSE;
 }
@@ -519,48 +588,72 @@ exclude_filesystem (GPtrArray *excluded_FSs, gchar *mountpoint, gchar *device)
 void
 disks_refresh(GPtrArray * pdisks, GPtrArray *excluded_FSs)
 {
-    /* using getmntent to get filesystems mount information */
+    /* using getmntent/getmntinfo to get filesystems mount information */
 
+#ifdef HAVE_GETMNTENT
     FILE * fmtab = NULL; /* file /etc/mtab */
     struct mntent * pmntent = NULL; /* struct for mnt info */
+#elif defined (HAVE_GETMNTINFO)
+    int i, nb_mounted_fs = 0;
+#endif
     struct statfs * pstatfs = NULL;
     gboolean exclude =  FALSE;
 
-    TRACE("Entering disks_refresh\n");
-
     t_mount_info * mount_info;
     t_disk * pdisk ;
 
+    TRACE("Entering disks_refresh");
+
     /* remove t_mount_info for all devices */
     disks_free_mount_info (pdisks);
 
+#ifdef HAVE_GETMNTENT
     /* allocate new struct statfs */
     pstatfs = g_new0 (struct statfs, 1);
 
     /* open file */
     fmtab = setmntent (MTAB, "r"); /* mtab file */
+#elif defined (HAVE_GETMNTINFO)
+    /* get mounted fs */
+    nb_mounted_fs = getmntinfo(&pstatfs,MNT_WAIT);
+#endif
 
     /* start looking for mounted devices */
+#ifdef HAVE_GETMNTENT
     for (pmntent=getmntent(fmtab); pmntent!=NULL; pmntent=getmntent(fmtab)) {
 
-        DBG (" have entry: %s on %s \n", pmntent->mnt_fsname, pmntent->mnt_dir );
+        DBG (" have entry: %s on %s", pmntent->mnt_fsname, pmntent->mnt_dir );
 
         statfs (pmntent->mnt_dir, pstatfs);
+#elif defined (HAVE_GETMNTINFO)
+    for (i = 0; i < nb_mounted_fs ; i++) {
+        DBG (" have entry: %s on %s : type %s", pstatfs[i].f_mntfromname, pstatfs[i].f_mntonname, pstatfs[i].f_fstypename );
+#endif
 
         /* if we got the stat and the block number is non-zero */
 
         /* get pointer on disk from pdisks */
         /* CHANGED to reflect change in disk_search */
+#ifdef HAVE_GETMNTENT
         pdisk = disks_search (pdisks, pmntent->mnt_dir);
+#elif defined (HAVE_GETMNTINFO)
+        pdisk = disks_search (pdisks, pstatfs[i].f_mntonname);
+#endif
         if (excluded_FSs!=NULL)
+#ifdef HAVE_GETMNTENT
             exclude = exclude_filesystem (excluded_FSs, pmntent->mnt_dir, pmntent->mnt_fsname);
+#elif defined (HAVE_GETMNTINFO)
+            exclude = exclude_filesystem (excluded_FSs, pstatfs[i].f_mntonname, pstatfs[i].f_mntfromname);
+#endif
 
         if (pdisk == NULL) { /* if disk is not found in pdisks */
 
             /* create a new struct t_disk and add it to pdisks */
             /* test for mnt_dir==none or neither block device nor NFS or system device */
             if ( exclude ||
+#ifdef HAVE_GETMNTENT
               g_ascii_strcasecmp(pmntent->mnt_dir, "none") == 0 ||
+              g_str_has_prefix(pmntent->mnt_fsname, "gvfs-fuse-daemon") ||
               !(g_str_has_prefix(pmntent->mnt_fsname, "/dev/") ||
               g_str_has_prefix(pmntent->mnt_type, "fuse") ||
               g_str_has_prefix(pmntent->mnt_type, "nfs") ||
@@ -568,25 +661,43 @@ disks_refresh(GPtrArray * pdisks, GPtrArray *excluded_FSs)
               g_str_has_prefix(pmntent->mnt_type, "cifs") ||
               g_str_has_prefix(pmntent->mnt_type, "shfs") ) ||
               g_str_has_prefix(pmntent->mnt_dir, "/sys/")
-
+#elif defined (HAVE_GETMNTINFO)
+              /* TODO: add support for more fs types on BSD */
+              g_ascii_strcasecmp(pstatfs[i].f_mntonname, "none") == 0 ||
+              !g_str_has_prefix(pstatfs[i].f_mntfromname, "/dev/") ||
+              g_str_has_prefix(pstatfs[i].f_fstypename, "nfs") ||
+              g_str_has_prefix(pstatfs[i].f_fstypename, "mfs")
+#endif
             ) continue;
 
             /* else have valid entry reflecting block device or NFS */
+#ifdef HAVE_GETMNTENT
             pdisk = disk_new (pmntent->mnt_fsname, pmntent->mnt_dir);
             pdisk->dc = disk_classify (pmntent->mnt_fsname, pmntent->mnt_dir);
+#elif defined (HAVE_GETMNTINFO)
+            pdisk = disk_new (pstatfs[i].f_mntfromname, pstatfs[i].f_mntonname);
+            pdisk->dc = disk_classify (pstatfs[i].f_mntfromname, pstatfs[i].f_mntonname);
+#endif
             g_ptr_array_add (pdisks, pdisk);
         }
 
         /* create new t_mount_info */
+#ifdef HAVE_GETMNTENT
         mount_info = mount_info_new_from_stat (pstatfs, pmntent->mnt_type,
                                                pmntent->mnt_dir);
+#elif defined (HAVE_GETMNTINFO)
+        mount_info = mount_info_new_from_stat (&pstatfs[i], pstatfs[i].f_fstypename,
+                                               pstatfs[i].f_mntonname);
+#endif
         /* add it to pdisk */
         pdisk->mount_info = mount_info ;
 
     } /* end for */
 
+#ifdef HAVE_GETMNTENT
     g_free (pstatfs);
     endmntent (fmtab); /* close file */
+#endif
 
     return;
 }
@@ -606,7 +717,7 @@ disk_classify (char *device, char *mountpoint)
             dc = REMOTE;
         }
     }
-    else if ( strstr(device, "cdrom") || strstr(device, "cdrw")
+    else if ( strstr(device, "cdrom") || strstr(device, "cdrw") || strstr(device, "cd")
                 || strstr(device, "dvd") || strstr(mountpoint, "cdrom")
                 || strstr(mountpoint, "cdrw") || strstr(mountpoint, "dvd")) {
         dc = CD_DVD;
@@ -627,25 +738,46 @@ disk_classify (char *device, char *mountpoint)
 gboolean
 disk_check_mounted (const char *disk)
 {
+#ifdef HAVE_GETMNTENT
     FILE *fmtab = NULL; /* file /etc/mtab */
     struct mntent *pmntent = NULL; /* struct for mnt info */
+#elif defined (HAVE_GETMNTINFO)
+    struct statfs * pstatfs = NULL;
+    int i, nb_mounted_fs = 0;
+#endif
     gboolean retval = FALSE;
 
+#ifdef HAVE_GETMNTENT
     /* open file */
     fmtab = setmntent (MTAB, "r"); /* mtab file */
+#elif defined (HAVE_GETMNTINFO)
+    /* get mounted fs */
+    nb_mounted_fs = getmntinfo(&pstatfs,MNT_WAIT);
+#endif
 
     /* start looking for mounted devices */
+#ifdef HAVE_GETMNTENT
     for (pmntent=getmntent(fmtab); pmntent!=NULL; pmntent=getmntent(fmtab))
+#elif defined (HAVE_GETMNTINFO)
+    for (i = 0; i < nb_mounted_fs ; i++)
+#endif
     {
+#ifdef HAVE_GETMNTENT
         if (strcmp(pmntent->mnt_dir, disk)==0 ||
             strcmp(pmntent->mnt_fsname, disk)==0 )
+#elif defined (HAVE_GETMNTINFO)
+        if (strcmp(pstatfs[i].f_mntonname, disk)==0 ||
+            strcmp(pstatfs[i].f_mntfromname, disk)==0 )
+#endif
         {
             retval = TRUE;
             break;
         }
     }
 
+#ifdef HAVE_GETMNTENT
     endmntent (fmtab); /* close file */
+#endif
 
     return retval;
 }
