@@ -3,6 +3,7 @@
 /*
 Copyright (C) 2005 Jean-Baptiste jb_dul@yahoo.com
 Copyright (C) 2005-2008 Fabian Nowak timystery@arcor.de.
+Copyright (C) 2012 Landry Breuil <landry@xfce.org>
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -19,13 +20,19 @@ You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
-#include "mount-plugin.h"
 
-static void
-on_message_dialog_response (GtkWidget *widget, gpointer *data)
-{
-    gtk_widget_destroy (widget);
-}
+#include <stdlib.h>
+
+#include "mount-plugin.h"
+#include <libxfce4ui/libxfce4ui.h>
+
+void format_LVM_name (const char *disk_device, gchar **formatted_diskname);
+
+#ifdef LIBXFCE4PANEL_CHECK_VERSION
+#if LIBXFCE4PANEL_CHECK_VERSION (4,9,0)
+#define HAS_PANEL_49
+#endif
+#endif
 
 
 static void
@@ -33,8 +40,6 @@ on_activate_disk_display (GtkWidget *widget, t_disk * disk)
 {
     t_mounter * mt;
     gboolean eject;
-    gchar *msg;
-    GtkWidget *my_dlg;
 
     TRACE ("enters on_activate_disk_display");
 
@@ -44,34 +49,10 @@ on_activate_disk_display (GtkWidget *widget, t_disk * disk)
 
     if (disk != NULL) {
         if (disk->mount_info != NULL) { /* disk is mounted */
-            int result = disk_umount (disk, mt->umount_command,
-                                      mt->message_dialog, eject);
-
-            if (mt->message_dialog && (!eject || result!=NONE) ) { /* popup dialog */
-
-                    msg = (gchar *) g_malloc (1024*sizeof(gchar));
-                    if (result==NONE)
-                        msg = _("The device \"%s\" should be removable safely "
-                                    "now.");
-                    else
-                        msg = _("An error occurred. The device should not be "
-                                    "removed!");
-
-                my_dlg = gtk_message_dialog_new (NULL,
-                        GTK_DIALOG_DESTROY_WITH_PARENT,
-                        GTK_MESSAGE_INFO,
-                        GTK_BUTTONS_OK,
-                        msg,
-                        disk->mount_point);
-
-                 g_signal_connect (my_dlg, "response",
-                        G_CALLBACK (on_message_dialog_response), my_dlg);
-                 gtk_widget_show (my_dlg);
-                    /* gtk_dialog_run (GTK_DIALOG (my_dlg)); */
-            }
+            disk_umount (disk, mt->umount_command, mt->message_dialog, eject, mt->use_sudo);
         }
         else { /* disk is not mounted */
-            disk_mount (disk, mt->on_mount_cmd, mt->mount_command, eject);
+            disk_mount (disk, mt->on_mount_cmd, mt->mount_command, eject, mt->use_sudo);
         }
     }
 
@@ -83,8 +64,10 @@ static void
 mounter_set_size (XfcePanelPlugin *plugin, int size, t_mounter *mt)
 {
    /* shrink the gtk button's image to new size -*/
-   gtk_widget_set_size_request (GTK_WIDGET(mt->button), size - 4, size - 4);
-
+#ifdef HAS_PANEL_49
+   size /= xfce_panel_plugin_get_nrows (plugin);
+#endif
+   gtk_widget_set_size_request (GTK_WIDGET(mt->button), size, size);
 }
 
 /**
@@ -116,6 +99,33 @@ format_LVM_name (const char *disk_device, gchar **formatted_diskname)
     *formatted_diskname = g_strdup_printf("LVM  %d:%d", volume, logvol);
 }
 
+/**
+ *  Set character sizes for all entries to maximum.
+ */
+static void
+disk_display_set_sizes (GPtrArray *array)
+{
+    unsigned int i, max_width_label_disk=0, max_width_label_mount_info=0, tmp;
+    t_disk_display *disk_display;
+
+    for (i=0; i<array->len; i++) {
+        disk_display= g_ptr_array_index (array, i); /* get the disk */
+        tmp = strlen(gtk_label_get_text(GTK_LABEL(disk_display->label_mount_info)));
+        if (tmp>max_width_label_mount_info)
+            max_width_label_mount_info = tmp;
+
+        tmp = strlen(gtk_label_get_text(GTK_LABEL(disk_display->label_disk)));
+        if (tmp>max_width_label_disk)
+            max_width_label_disk = tmp;
+    }
+
+    for (i=0; i<array->len; i++) {
+        disk_display = g_ptr_array_index (array, i); /* get the disk */
+        gtk_label_set_width_chars(GTK_LABEL(disk_display->label_disk), max_width_label_disk);
+        gtk_label_set_width_chars(GTK_LABEL(disk_display->label_mount_info), max_width_label_mount_info);
+    }
+
+}
 
 /**
  * Create a new t_disk_display from t_disk infos.
@@ -146,22 +156,22 @@ disk_display_new (t_disk *disk, t_mounter *mounter)
             formatted_diskname = g_strdup(disk->device);
 
         if (mounter->exclude_devicenames)
-        dd->label_disk = gtk_label_new (disk->mount_point);
-    else
-        dd->label_disk = gtk_label_new (g_strconcat(formatted_diskname, " -> ",
+            dd->label_disk = gtk_label_new (disk->mount_point);
+        else
+            dd->label_disk = gtk_label_new (g_strconcat(formatted_diskname, " -> ",
                                         disk->mount_point, NULL));
 
-    g_free (formatted_diskname);
+        g_free (formatted_diskname);
 
         /*change to uniform label size*/
-        gtk_label_set_width_chars(GTK_LABEL(dd->label_disk), 32);
+        /*gtk_label_set_width_chars(GTK_LABEL(dd->label_disk), 32); */
         /* gtk_label_set_justify(GTK_LABEL(dd->label_disk),GTK_JUSTIFY_LEFT); */
         gtk_misc_set_alignment(GTK_MISC(dd->label_disk),0.0, 0.5);
         gtk_box_pack_start(GTK_BOX(dd->hbox),dd->label_disk,FALSE,TRUE,0);
 
         dd->label_mount_info = gtk_label_new("");
         /*change to uniform label size*/
-        gtk_label_set_width_chars(GTK_LABEL(dd->label_mount_info),25);
+        /* gtk_label_set_width_chars(GTK_LABEL(dd->label_mount_info),25); */
         gtk_label_set_use_markup(GTK_LABEL(dd->label_mount_info),TRUE);
         /* gtk_label_set_justify (GTK_LABEL(dd->label_mount_info),
                                GTK_JUSTIFY_RIGHT); */
@@ -181,14 +191,16 @@ disk_display_new (t_disk *disk, t_mounter *mounter)
 
 
 static void
-disk_display_refresh (t_disk_display * disk_display,
-                                 t_mount_info * mount_info)
+disk_display_refresh (t_disk_display * disk_display)
 {
+    t_mount_info * mount_info;
     char * text;
     char * used;
     char * size;
     char * avail;
 
+    mount_info = disk_display->disk->mount_info;
+
     TRACE("enters disk_display_refresh");
 
     if (disk_display != NULL)
@@ -210,7 +222,7 @@ disk_display_refresh (t_disk_display * disk_display,
                      ((gdouble)mount_info->percent / 100) );
             gtk_progress_bar_set_text (
                      GTK_PROGRESS_BAR(disk_display->progress_bar),
-                     g_strdup_printf ("%d",mount_info->percent));
+                     g_strdup_printf ("%d%%",mount_info->percent));
             gtk_widget_show (GTK_WIDGET(disk_display->progress_bar));
         }
         else /* mount_info == NULL */
@@ -255,29 +267,28 @@ mounter_free (XfcePanelPlugin *plugin, t_mounter *mounter)
 static void
 mounter_data_new (t_mounter *mt)
 {
-    TRACE ("enters mounter_data_new");
-
-    int i, res;
+    unsigned int i;
     t_disk * disk;
     t_disk_display * disk_display;
-    GPtrArray *array =  NULL;
+    GPtrArray *array =  NULL, *disk_displays = NULL;
     char *dev_mp; /* device or mountpoint */
-    gboolean removed_device;
 
-    /*get static infos from /etc/fstab */
-    mt->pdisks = disks_new (mt->include_NFSs);
+    TRACE ("enters mounter_data_new");
+
+    /* get static infos from /etc/fstab */
+    mt->pdisks = disks_new (mt->include_NFSs, &(mt->showed_fstab_dialog));
 
     /* remove unwanted file systems from list */
     if (mt->exclude_FSs) {
         array = g_ptr_array_new();
-        DBG("excluded_filesystems=%s\n", mt->excluded_filesystems);
-        res = seperate_list(array, mt->excluded_filesystems);
+        DBG("excluded_filesystems=%s", mt->excluded_filesystems);
+        seperate_list(array, mt->excluded_filesystems);
         for (i=0; i<array->len; i++) {
             dev_mp = (char*) g_ptr_array_index(array, i);
             if ( strstr(dev_mp , "/dev") )
-                removed_device = disks_remove_device(mt->pdisks, dev_mp);
+                disks_remove_device(mt->pdisks, dev_mp);
             else
-                removed_device = disks_remove_mountpoint (mt->pdisks, dev_mp);
+                disks_remove_mountpoint (mt->pdisks, dev_mp);
         }
     }
 
@@ -286,14 +297,15 @@ mounter_data_new (t_mounter *mt)
 
     /* menu with menu_item */
     mt->menu = gtk_menu_new ();
-
+    disk_displays =g_ptr_array_new();
     for (i=0; i < mt->pdisks->len; i++)
     {
-        disk = g_ptr_array_index (mt->pdisks,i); /* get the disk */
-        disk_display = disk_display_new (disk,mt); /* creates a disk_display */
-
+        disk = g_ptr_array_index (mt->pdisks, i); /* get the disk */
+        disk_display = disk_display_new (disk, mt); /* creates a disk_display */
+        disk_display->disk = disk;
+        g_ptr_array_add(disk_displays, disk_display);
         /* fill in mount infos */
-        disk_display_refresh (disk_display,disk->mount_info);
+        disk_display_refresh (disk_display);
 
         /* add the menu_item to the menu */
         gtk_menu_shell_append (GTK_MENU_SHELL(mt->menu),
@@ -301,21 +313,7 @@ mounter_data_new (t_mounter *mt)
     }
     gtk_widget_show_all(mt->menu);
 
-    mt->icon = PACKAGE_DATA_DIR"/icons/hicolor/scalable/apps/xfce-mount.svg";
-    mt->mount_command = DEFAULT_MOUNT_COMMAND;
-    mt->umount_command = DEFAULT_UMOUNT_COMMAND;
-
-    mt->excluded_filesystems = "";
-
-    mt->message_dialog = FALSE;
-
-    mt->include_NFSs = FALSE;
-
-    mt->exclude_FSs = FALSE;
-
-    mt->eject_drives = FALSE;
-
-    mt->exclude_devicenames = FALSE;
+    disk_display_set_sizes(disk_displays);
 
     TRACE ("leaves mounter_data_new");
 
@@ -329,31 +327,9 @@ mounter_refresh (t_mounter * mt)
     TRACE ("enters mounter_refresh");
 
     mounter_data_free (mt);
-    gchar *mount = g_strdup (mt->mount_command);
-    gchar *umount = g_strdup (mt->umount_command);
-    gchar *icon = g_strdup (mt->icon);
-    gchar *excl_filesystems = g_strdup (mt->excluded_filesystems);
-    DBG ("Changed icon value from '%s' to '%s'.\n", mt->icon, icon);
-    gboolean msg_dlg = mt->message_dialog;
-    gboolean incl_NFSs = mt->include_NFSs;
-    gboolean excl_FSs = mt->exclude_FSs;
-    gboolean excl_DNs = mt->exclude_devicenames;
-    gboolean eject = mt->eject_drives;
-
     mounter_data_new (mt);
-    mt->icon = g_strdup (icon);
-    DBG ("Changed icon value from '%s' to '%s'.\n", icon, mt->icon);
-    mt->mount_command = g_strdup (mount);
-    mt->umount_command = g_strdup (umount);
-    mt->excluded_filesystems = g_strdup (excl_filesystems);
-    mt->message_dialog = msg_dlg;
-    mt->include_NFSs = incl_NFSs;
-    mt->exclude_FSs = excl_FSs;
-    mt->exclude_devicenames = excl_DNs;
-    mt->eject_drives = eject;
 
     TRACE ("leaves mounter_refresh");
-
 }
 
 
@@ -377,57 +353,59 @@ on_button_press (GtkWidget *widget, GdkEventButton *event, t_mounter *mounter)
 static void
 mounter_read_config (XfcePanelPlugin *plugin, t_mounter *mt)
 {
-    TRACE ("enter read_config");
-
-    const char *value;
+    gchar *icon;
     char *file;
     XfceRc *rc;
+    TRACE ("enter read_config");
 
     if ( !( file = xfce_panel_plugin_lookup_rc_file (plugin) ) )
         return;
-
+    DBG("going to read config from %s", file);
     rc = xfce_rc_simple_open (file, TRUE);
     g_free (file);
 
-    if ( (value = xfce_rc_read_entry(rc, "on_mount_cmd", NULL)) )
-      mt->on_mount_cmd = g_strdup (value);
+    if (mt->icon != NULL) g_free(mt->icon);
+    if (mt->on_mount_cmd != NULL) g_free(mt->on_mount_cmd);
+    if (mt->mount_command != NULL) g_free(mt->mount_command);
+    if (mt->umount_command != NULL) g_free(mt->umount_command);
+    if (mt->excluded_filesystems != NULL) g_free(mt->excluded_filesystems);
 
-    if ( (value = xfce_rc_read_entry(rc, "icon", NULL)) )
-        mt->icon = g_strdup (value);
+    icon = g_strdup_printf ("%s/icons/hicolor/scalable/apps/xfce-mount.svg", PACKAGE_DATA_DIR );
+    mt->icon = g_strdup(xfce_rc_read_entry(rc, "icon", icon));
+    g_free(icon);
+
+    mt->on_mount_cmd = g_strdup(xfce_rc_read_entry(rc, "on_mount_cmd", ""));
+    mt->mount_command = g_strdup(xfce_rc_read_entry(rc, "mount_command", DEFAULT_MOUNT_COMMAND));
+    mt->umount_command = g_strdup(xfce_rc_read_entry(rc, "umount_command", DEFAULT_UMOUNT_COMMAND));
+    mt->excluded_filesystems = g_strdup(xfce_rc_read_entry(rc, "excluded_filesystems", ""));
+
+    mt->use_sudo = xfce_rc_read_bool_entry(rc, "use_sudo", FALSE);
+
+    /* before 0.6.0 those booleans were stored as string "1".. handle/merge legacy configs */
+    if (xfce_rc_has_entry(rc, "message_dialog"))
+        mt->message_dialog = atoi(xfce_rc_read_entry(rc, "message_dialog", NULL));
     else
-        mt->icon = g_strdup_printf (
-                "%s/icons/hicolor/scalable/apps/xfce-mount.svg",
-                PACKAGE_DATA_DIR );
+        mt->message_dialog = xfce_rc_read_bool_entry(rc, "show_message_dialog", FALSE);
 
-    if ( (value = xfce_rc_read_entry (rc, "mount_command", NULL)) )
-        mt->mount_command = g_strdup (value);
+    if (xfce_rc_has_entry(rc, "include_NFSs"))
+        mt->include_NFSs = atoi(xfce_rc_read_entry(rc, "include_NFSs", NULL));
     else
-        mt->mount_command = g_strdup (DEFAULT_MOUNT_COMMAND);
+        mt->include_NFSs = xfce_rc_read_bool_entry(rc, "include_networked_filesystems", FALSE);
 
-    if ( (value = xfce_rc_read_entry (rc, "umount_command", NULL)) )
-        mt->umount_command = g_strdup (value);
+    if (xfce_rc_has_entry(rc, "exclude_FSs"))
+        mt->exclude_FSs = atoi(xfce_rc_read_entry(rc, "exclude_FSs", NULL));
     else
-        mt->umount_command = g_strdup (DEFAULT_UMOUNT_COMMAND);
+        mt->exclude_FSs = xfce_rc_read_bool_entry(rc, "exclude_selected_filesystems", FALSE);
 
-    if ( (value = xfce_rc_read_entry (rc, "excluded_filesystems", NULL)) )
-        mt->excluded_filesystems = g_strdup (value);
+    if (xfce_rc_has_entry(rc, "exclude_devicenames"))
+        mt->exclude_devicenames = atoi(xfce_rc_read_entry(rc, "exclude_devicenames", NULL));
     else
-        mt->excluded_filesystems = g_strdup ("");
+        mt->exclude_devicenames = xfce_rc_read_bool_entry(rc, "exclude_devicenames_in_menu", FALSE);
 
-    if ( (value = xfce_rc_read_entry(rc, "message_dialog", NULL)) )
-        mt->message_dialog = atoi (value);
-
-    if ( (value = xfce_rc_read_entry(rc, "include_NFSs", NULL)) )
-        mt->include_NFSs= atoi (value);
-
-    if ( (value = xfce_rc_read_entry(rc, "exclude_FSs", NULL)) )
-        mt->exclude_FSs= atoi (value);
-
-    if ( (value = xfce_rc_read_entry(rc, "exclude_devicenames", NULL)) )
-        mt->exclude_devicenames= atoi (value);
-
-    if ( (value = xfce_rc_read_entry(rc, "eject_drives", NULL)) )
-        mt->eject_drives= atoi (value);
+    if (xfce_rc_has_entry(rc, "eject_drives"))
+        mt->eject_drives = atoi(xfce_rc_read_entry(rc, "eject_drives", NULL));
+    else
+        mt->eject_drives = xfce_rc_read_bool_entry(rc, "eject_cddrives", FALSE);
 
     xfce_rc_close (rc);
 
@@ -438,50 +416,33 @@ mounter_read_config (XfcePanelPlugin *plugin, t_mounter *mt)
 static void
 mounter_write_config (XfcePanelPlugin *plugin, t_mounter *mt)
 {
-    TRACE ("enter write_config");
-
-     XfceRc *rc;
+    XfceRc *rc;
     char *file;
+    TRACE ("enter write_config");
 
     if (!(file = xfce_panel_plugin_save_location (plugin, TRUE)))
         return;
 
     /* int res = */ unlink (file);
 
+    DBG("going to write config to %s", file);
     rc = xfce_rc_simple_open (file, FALSE);
     g_free (file);
 
     if (!rc)
         return;
 
-    xfce_rc_write_entry (rc, "on_mount_cmd",
-         mt->on_mount_cmd ? mt->on_mount_cmd : "");
-
-    if ( strcmp(mt->mount_command, DEFAULT_MOUNT_COMMAND)!=0 )
-        xfce_rc_write_entry (rc, "mount_command", mt->mount_command);
-
-    if ( strcmp(mt->umount_command, DEFAULT_UMOUNT_COMMAND)!=0 )
-        xfce_rc_write_entry (rc, "umount_command", mt->umount_command);
-
-    if ( strlen(mt->excluded_filesystems)!=0 )
-        xfce_rc_write_entry (rc, "excluded_filesystems", mt->excluded_filesystems);
-
-    if ( mt->message_dialog==1 )
-        xfce_rc_write_entry (rc, "message_dialog", "1");
-
-    if ( mt->include_NFSs==1 )
-        xfce_rc_write_entry (rc, "include_NFSs", "1");
-
-    if ( mt->exclude_FSs==1 )
-        xfce_rc_write_entry (rc, "exclude_FSs", "1");
-
-    if ( mt->exclude_devicenames==1 )
-        xfce_rc_write_entry (rc, "exclude_devicenames", "1");
-
-    if ( mt->eject_drives==1 )
-        xfce_rc_write_entry (rc, "eject_drives", "1");
-
+    xfce_rc_write_entry (rc, "on_mount_cmd", mt->on_mount_cmd);
+    xfce_rc_write_entry (rc, "mount_command", mt->mount_command);
+    xfce_rc_write_entry (rc, "umount_command", mt->umount_command);
+    xfce_rc_write_entry (rc, "excluded_filesystems", mt->excluded_filesystems);
     xfce_rc_write_entry (rc, "icon", mt->icon);
+    xfce_rc_write_bool_entry (rc, "show_message_dialog", mt->message_dialog);
+    xfce_rc_write_bool_entry (rc, "include_networked_filesystems", mt->include_NFSs);
+    xfce_rc_write_bool_entry (rc, "exclude_selected_filesystems", mt->exclude_FSs);
+    xfce_rc_write_bool_entry (rc, "exclude_devicenames_in_menu", mt->exclude_devicenames);
+    xfce_rc_write_bool_entry (rc, "eject_cddrives", mt->eject_drives);
+    xfce_rc_write_bool_entry (rc, "use_sudo", mt->use_sudo);
 
     xfce_rc_close (rc);
 
@@ -492,16 +453,23 @@ mounter_write_config (XfcePanelPlugin *plugin, t_mounter *mt)
 static t_mounter *
 create_mounter_control (XfcePanelPlugin *plugin)
 {
-    TRACE ("enters create_mounter_control");
-
     t_mounter *mounter;
+    TRACE ("enters create_mounter_control");
 
     mounter = g_new0(t_mounter,1);
 
-    /* default mount command */
-    mounter->on_mount_cmd = NULL;
-
-    mounter->icon = NULL;
+    /* default configuration values for when no configuration is found */
+    mounter->icon = g_strdup(PACKAGE_DATA_DIR"/icons/hicolor/scalable/apps/xfce-mount.svg");
+    mounter->mount_command = g_strdup(DEFAULT_MOUNT_COMMAND);
+    mounter->umount_command = g_strdup(DEFAULT_UMOUNT_COMMAND);
+    mounter->on_mount_cmd = g_strdup("");
+    mounter->excluded_filesystems = g_strdup("");
+    mounter->message_dialog = FALSE;
+    mounter->include_NFSs = FALSE;
+    mounter->exclude_FSs = FALSE;
+    mounter->eject_drives = FALSE;
+    mounter->exclude_devicenames = FALSE;
+    mounter->use_sudo = FALSE;
 
     mounter->plugin = plugin;
 
@@ -518,8 +486,10 @@ create_mounter_control (XfcePanelPlugin *plugin)
 
     g_assert (mounter->icon!=NULL);
 
-    mounter->button_pb = gdk_pixbuf_new_from_file (mounter->icon, NULL);
-    mounter->button = xfce_iconbutton_new_from_pixbuf (mounter->button_pb);
+    mounter->button = gtk_button_new ();
+    mounter->image = xfce_panel_image_new_from_source (mounter->icon);
+    gtk_widget_show(mounter->image);
+    gtk_container_add (GTK_CONTAINER(mounter->button), mounter->image);
     gtk_button_set_relief (GTK_BUTTON(mounter->button), GTK_RELIEF_NONE);
 
     gtk_tooltips_set_tip (tooltips, GTK_WIDGET(mounter->button), _("devices"),
@@ -544,21 +514,16 @@ free_mounter_dialog(GtkWidget * widget, t_mounter_dialog * md)
 static void
 mounter_apply_options (t_mounter_dialog *md)
 {
+    gboolean incl_NFSs, excl_FSs;
+    t_mounter * mt = md->mt;
     TRACE ("enters mounter_apply_options");
 
-    t_mounter * mt = md->mt;
 
-    const char * tmp;
-    tmp = gtk_entry_get_text (GTK_ENTRY(md->string_cmd));
+    incl_NFSs = mt->include_NFSs;
+    excl_FSs = mt->exclude_FSs;
 
-    gboolean incl_NFSs = mt->include_NFSs;
-    gboolean excl_FSs = mt->exclude_FSs;
-
-    g_free (mt->on_mount_cmd);
-    if (tmp && *tmp)
-        mt->on_mount_cmd = g_strdup (tmp);
-    else
-        mt->on_mount_cmd = NULL;
+    mt->on_mount_cmd = g_strdup ( gtk_entry_get_text
+                          (GTK_ENTRY(md->string_cmd)) );
 
     if ( gtk_toggle_button_get_active
             (GTK_TOGGLE_BUTTON(md->specify_commands)) ) {
@@ -584,6 +549,9 @@ mounter_apply_options (t_mounter_dialog *md)
     mt->eject_drives = gtk_toggle_button_get_active
             (GTK_TOGGLE_BUTTON(md->show_eject_drives));
 
+    mt->use_sudo = gtk_toggle_button_get_active
+            (GTK_TOGGLE_BUTTON(md->show_use_sudo));
+
     mt->exclude_FSs = gtk_toggle_button_get_active
             (GTK_TOGGLE_BUTTON(md->show_exclude_FSs));
 
@@ -605,8 +573,10 @@ mounter_apply_options (t_mounter_dialog *md)
                    "%s/icons/hicolor/scalable/apps/xfce-mount.svg",
                    PACKAGE_DATA_DIR );
 
-       mt->button_pb = gdk_pixbuf_new_from_file (mt->icon, NULL);
-       xfce_iconbutton_set_pixbuf (XFCE_ICONBUTTON(mt->button), mt->button_pb);
+    gtk_container_remove(GTK_CONTAINER(mt->button), mt->image);
+    mt->image = xfce_panel_image_new_from_source (mt->icon);
+    gtk_widget_show(mt->image);
+    gtk_container_add (GTK_CONTAINER(mt->button), mt->image);
 
     TRACE ("leaves mounter_apply_options");
 }
@@ -670,39 +640,44 @@ exlude_FSs_toggled (GtkWidget *widget, t_mounter_dialog *md)
 }
 
 
-static gboolean
+/*static gboolean
 exclude_devicenames_toggled (GtkWidget *widget, t_mounter_dialog *md)
 {
 
 
     return TRUE;
-}
+}*/
 
 static void
 mounter_create_options (XfcePanelPlugin *plugin, t_mounter *mt)
 {
 
+    GtkWidget *dlg;
+    GtkWidget *vbox;
+    GtkWidget *_eventbox;
+    GtkWidget *_label;
+    GtkWidget *_vbox, *_vbox2;
+    GtkWidget *_hbox;
+    GtkTooltips *tip;
+    GtkWidget *_notebook;
+    t_mounter_dialog * md;
+    gboolean set_active;
+
     TRACE ("enters mounter_create_options");
 
     xfce_panel_plugin_block_menu (plugin);
 
-    GtkWidget *dlg, *header;
-    dlg = gtk_dialog_new_with_buttons (_("Edit Properties"),
+    dlg = xfce_titled_dialog_new_with_buttons(
+                _("Mount Plugin"),
                 GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (plugin))),
                 GTK_DIALOG_DESTROY_WITH_PARENT | GTK_DIALOG_NO_SEPARATOR,
                 GTK_STOCK_CLOSE, GTK_RESPONSE_OK, NULL);
 
+    xfce_titled_dialog_set_subtitle (XFCE_TITLED_DIALOG (dlg), _("Properties"));
+    gtk_window_set_icon_name(GTK_WINDOW(dlg),"drive-harddisk");
+
     gtk_container_set_border_width (GTK_CONTAINER (dlg), 2);
 
-    header = xfce_create_header (NULL, _("Mount devices"));
-    gtk_widget_set_size_request (GTK_BIN (header)->child, -1, 32);
-    gtk_container_set_border_width (GTK_CONTAINER (header), BORDER - 2);
-    gtk_widget_show (header);
-    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dlg)->vbox), header, FALSE,
-                        TRUE, 0);
-
-    GtkWidget *vbox;
-    t_mounter_dialog * md;
 
     md = g_new0 (t_mounter_dialog, 1);
 
@@ -712,15 +687,6 @@ mounter_create_options (XfcePanelPlugin *plugin, t_mounter *mt)
 
     vbox = GTK_DIALOG (dlg)->vbox;
 
-    /* "local" variables */
-    GtkWidget *_eventbox;
-    GtkWidget *_label;
-    GtkWidget *_vbox, *_vbox2;
-    GtkWidget *_hbox;
-    GtkTooltips *tip;
-    GtkWidget *_notebook;
-
-
     _notebook = gtk_notebook_new ();
     gtk_widget_show (_notebook);
     gtk_container_border_width (GTK_CONTAINER(_notebook), BORDER);
@@ -785,6 +751,23 @@ mounter_create_options (XfcePanelPlugin *plugin, t_mounter *mt)
     gtk_container_border_width (GTK_CONTAINER(_vbox), BORDER);
     gtk_widget_show (_vbox);
 
+        /* use sudo checkbox */
+    _eventbox = gtk_event_box_new ();
+    gtk_box_pack_start (GTK_BOX (_vbox), GTK_WIDGET(_eventbox),
+                        FALSE, FALSE, 0);
+    gtk_widget_show (_eventbox);
+    gtk_tooltips_set_tip ( GTK_TOOLTIPS(tip), _eventbox,
+        _("Activate this option to use sudo(8) to run mount/umount commands. Needs to be configured without password."),
+        NULL );
+
+    md->show_use_sudo = gtk_check_button_new_with_mnemonic (
+                                _("Use _sudo") );
+
+    gtk_widget_show (md->show_use_sudo);
+    gtk_container_add (GTK_CONTAINER (_eventbox), md->show_use_sudo );
+    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(md->show_use_sudo),
+                                mt->use_sudo);
+
         /* After-mount command */
     _eventbox = gtk_event_box_new ();
     gtk_box_pack_start (GTK_BOX (_vbox), GTK_WIDGET(_eventbox),
@@ -793,7 +776,7 @@ mounter_create_options (XfcePanelPlugin *plugin, t_mounter *mt)
     gtk_tooltips_set_tip ( GTK_TOOLTIPS(tip), _eventbox,
         _("This command will be executed after mounting the device with the "
         "mount point of the device as argument.\n"
-        "If you are unsure what to insert, try \"thunar %m\".\n"
+        "If you are unsure what to insert, try \"exo-open %m\".\n"
         "'%d' can be used to specify the device, '%m' for the mountpoint."),
         NULL);
 
@@ -830,7 +813,7 @@ mounter_create_options (XfcePanelPlugin *plugin, t_mounter *mt)
 
     md->specify_commands = gtk_check_button_new_with_mnemonic (
                                 _("_Custom commands") );
-    gboolean set_active =
+    set_active =
         ( strcmp(mt->mount_command, DEFAULT_MOUNT_COMMAND)!=0 ||
         strcmp(mt->umount_command, DEFAULT_UMOUNT_COMMAND)!=0 ) ?
                 TRUE : FALSE;
@@ -870,7 +853,7 @@ mounter_create_options (XfcePanelPlugin *plugin, t_mounter *mt)
                     GTK_FILL, GTK_SHRINK, 0, 0);
 
     md->string_mount_command = gtk_entry_new ();
-    DBG("mt->mount_command: %s\n", mt->mount_command);
+    DBG("mt->mount_command: %s", mt->mount_command);
     gtk_entry_set_text (GTK_ENTRY(md->string_mount_command ),
                     g_strdup(mt->mount_command ));
     gtk_widget_show (md->string_mount_command );
@@ -996,21 +979,40 @@ mounter_create_options (XfcePanelPlugin *plugin, t_mounter *mt)
     TRACE ("leaves mounter_create_options");
 }
 
+static void
+mounter_show_about(XfcePanelPlugin *plugin, t_mounter *mt)
+{
+   GdkPixbuf *icon;
+   const gchar *auth[] = { "Jean-Baptiste Dulong",
+                           "Fabian Nowak <timystery@arcor.de>",
+                           "Landry Breuil <landry at xfce.org>", NULL };
+   icon = xfce_panel_pixbuf_from_source("drive-harddisk", NULL, 32);
+   gtk_show_about_dialog(NULL,
+      "logo", icon,
+      "license", xfce_get_license_text (XFCE_LICENSE_TEXT_GPL),
+      "version", PACKAGE_VERSION,
+      "program-name", PACKAGE_NAME,
+      "comments", _("Show partitions/devices and allow to mount/unmount them"),
+      "website", "http://goodies.xfce.org/projects/panel-plugins/xfce4-mount-plugin",
+      "copyright", _("Copyright (c) 2005-2012\n"),
+      "authors", auth, NULL);
+
+   if(icon)
+      g_object_unref(G_OBJECT(icon));
+}
 
 static void
 mount_construct (XfcePanelPlugin *plugin)
 {
-    xfce_textdomain (GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR, "UTF-8");
-
     t_mounter *mounter;
 
+    xfce_textdomain (GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR, "UTF-8");
+
     mounter = create_mounter_control (plugin);
 
-    mounter_read_config (plugin, mounter);
-
-    mounter->button_pb = gdk_pixbuf_new_from_file (mounter->icon, NULL);
-       xfce_iconbutton_set_pixbuf (XFCE_ICONBUTTON(mounter->button),
-                                   mounter->button_pb);
+#ifdef HAS_PANEL_49
+    xfce_panel_plugin_set_small (plugin, TRUE);
+#endif
 
     g_signal_connect (plugin, "free-data", G_CALLBACK (mounter_free), mounter);
 
@@ -1021,6 +1023,9 @@ mount_construct (XfcePanelPlugin *plugin)
     g_signal_connect (plugin, "configure-plugin",
                       G_CALLBACK (mounter_create_options), mounter);
 
+    xfce_panel_plugin_menu_show_about(plugin);
+    g_signal_connect (plugin, "about", G_CALLBACK (mounter_show_about), mounter);
+
     g_signal_connect (plugin, "size-changed", G_CALLBACK (mounter_set_size),
                          mounter);
 
@@ -1030,4 +1035,4 @@ mount_construct (XfcePanelPlugin *plugin)
 
 }
 
-XFCE_PANEL_PLUGIN_REGISTER_EXTERNAL (mount_construct);
+XFCE_PANEL_PLUGIN_REGISTER (mount_construct);
