@@ -27,23 +27,13 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 #include <config.h>
 #endif
 
-#ifdef DEBUG
-#undef DEBUG
-#endif
-#ifdef DEBUG_TRACE
-#undef DEBUG_TRACE
-#endif
-
 #include <gtk/gtk.h>
 #include <libxfce4panel/xfce-panel-plugin.h>
-#include <libxfce4util/libxfce4util.h>
-#include <libxfcegui4/libxfcegui4.h>
 
 #include <string.h>
 
 #include "devices.h"
 #include "helpers.h"
-#include "icons.h"
 
 #define APP_NAME N_("Mount Plugin")
 
@@ -52,7 +42,7 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 #define DEFAULT_MOUNT_COMMAND "mount %d"
 #define DEFAULT_UMOUNT_COMMAND "umount %d"
 
-#define DEFAULT_ICON PACKAGE_DATA_DIR"/icons/hicolor/scalable/apps/xfce-mount.svg"
+#define DEFAULT_ICON PACKAGE_DATA_DIR "/icons/hicolor/scalable/apps/xfce-mount.svg"
 
 static GtkTooltips *tooltips = NULL;
 
@@ -73,8 +63,10 @@ typedef struct
     gboolean exclude_FSs;
     gboolean exclude_devicenames;
     gboolean eject_drives;
+    gboolean use_sudo;
+    gboolean showed_fstab_dialog;
     GtkWidget *button;
-    GdkPixbuf *button_pb;
+    GtkWidget *image;
     GtkWidget *menu;
     GPtrArray *pdisks; /* contains pointers to struct t_disk */
 } t_mounter;
@@ -91,6 +83,7 @@ typedef struct
     GtkWidget * label_disk;
     GtkWidget * label_mount_info;
     GtkWidget * progress_bar;
+    t_disk *disk; /* People, learn to program in an object-oriented way! */
 } t_disk_display;
 
 
@@ -113,6 +106,7 @@ typedef struct
     GtkWidget *show_include_NFSs;
     GtkWidget *show_exclude_FSs;
     GtkWidget *show_eject_drives;
+    GtkWidget *show_use_sudo;
     GtkWidget *show_exclude_devicenames;
     GtkWidget *string_excluded_filesystems;
 }
